#################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009, 2010
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################

#################################################################################
# Conditional Distributions
#################################################################################

rgarch.test101 = function()
{
	cat("\nrgarch-->test10-1: Conditional Distributions\n")
	data(sp500ret)
	# we use the simple GARCH model for the comparisons
	spec = vector(mode = "list", length = 9)
	dist = c("norm", "snorm", "std", "sstd", "ged", "sged", "nig", "ghyp", "jsu")
	for(i in 1:9){
		spec[[i]] = ugarchspec(mean.model = list(armaOrder = c(0,0)),
				distribution.model = dist[i])
	}
	fit = vector(mode = "list", length = 9)
	for(i in 1:9){
		fit[[i]] = ugarchfit(data = sp500ret[,1], spec = spec[[i]], fit.control = list(scale = 1))
	}
	gofv = as.data.frame(matrix(NA, ncol = 9, nrow = 4))
	gofp = as.data.frame(matrix(NA, ncol = 9, nrow = 4))
	colnames(gofv) = colnames(gofp) = dist
	rownames(gofv) = rownames(gofp) = c("20", "30", "40","50")
	for(i in 1:9){
		 tmp = gof(fit[[i]],c(20, 30, 40, 50))
		 gofv[1:4,i] = tmp[,2]
		 gofp[1:4,i] = tmp[,3]
	}
	options(width=120)
	zz <- file("test10-1.txt", open="wt")
	sink(zz)
	cat("\nAdjusted Pearson Goodness-of-Fit Test:")
	cat("\n-------------------------------------\n")
	cat("Bins: 20, 30, 40, 50:\n")
	cat("\nstatistic\n")
	print(gofv, digits = 3)
	cat("\np-values\n")
	print(gofp, digits = 3)
	sink(type="message")
	sink()
	close(zz)
	
	postscript("test10-1.eps", width = 12, height = 12)
	par(mfrow = c(3,3))
	for(i in 1:9){
		xseries = fit[[i]]@fit$z
		xlim 	= NULL
		Values 	= as.vector(xseries)
		xlim 	= c(min(xseries), max(xseries))
		result 	= hist(x = Values, col = "grey", border = "white",
				breaks = "Scott", main = 
						paste("Empirical Density of Standardized Residuals (", dist[i], ")",sep=""),
				xlim = xlim, ylim=c(0,0.6), probability = TRUE, ylab="Probability", 
				cex.main = 0.8)
		s = result$breaks
		box()
		distribution = fit[[i]]@model$distribution
		fd 	= .getDensity(distribution)
		Names 	= fit[[i]]@model$modelnames
		if(any(substr(Names, 1, 7)=="dlambda")) dlambda = fit[[i]]@fit$coef["dlambda"] else dlambda = 0
		if(any(substr(Names, 1, 5)=="shape")) shape = fit[[i]]@fit$coef["shape"] else shape = 4
		if(any(substr(Names, 1, 4)=="skew")) skew = fit[[i]]@fit$coef["skew"] else skew = 4
		y	= fd(z = s, hh = 1, lambda = dlambda, skew = skew, shape = shape)
		lines(s, y, lwd = 2, col = i+1)
	}
	dev.off()
}

rgarch.test102 = function()
{
	cat("\nrgarch-->test10-2: GHYP vs NIG (Likelihood Ratio Test of Restriction lambda = -0.5)\n")
	data(sp500ret)
	# run the unrestricted model first to compute
	spec.gh = ugarchspec(mean.model = list(armaOrder = c(1,1)), distribution.model = "ghyp")
	fit.gh = ugarchfit(data = sp500ret[,1], spec.gh, solver = "solnp", fit.control = list(fixed.se = 0, scale = 0))
	
	spec.nig = ugarchspec(mean.model = list(armaOrder = c(1,1)), distribution.model = "ghyp", 
			fixed.pars = list(dlambda = -0.5))
	fit.nig = ugarchfit(data = sp500ret[,1], spec.nig, solver = "solnp", fit.control = list(fixed.se = 1, scale = 0))
	
	spec.hyp = ugarchspec(mean.model = list(armaOrder = c(1,1)), distribution.model = "ghyp", 
			fixed.pars = list(dlambda = 1))
	fit.hyp = ugarchfit(data = sp500ret[,1], spec.hyp, solver = "solnp", fit.control = list(fixed.se = 1, scale = 0))
	
	sc = apply(fit.nig@fit$scores, 2, "sum")
	lmtest.v = (t(sc)%*%solve(t(fit.nig@fit$scores)%*%fit.nig@fit$scores)%*%(sc))
	# equivalent to summary(lm(ones~scores-1))$r.squared*N
	lmtest.pvalue = 1 - pchisq(lmtest.v, 1)
	options(width = 120)
	zz <- file("test10-2.txt", open="wt")
	sink(zz)
	cat("\nNull Hypothesis of NIG distribution (lambda = -0.5 in GHYP family):\n")
	cat("\nUnrestricted conditional GHYP fit:\n")
	cat(c(paste("lambda: "), round(as.numeric(coef(fit.gh)["dlambda"]), 3)))
	cat("\n\n")
	cat("\nLM Test (Restriction: lambda = -0.5)\n\n")
	print(data.frame(LM.stat = lmtest.v, dof = 1, p.value = lmtest.pvalue))
	if(lmtest.pvalue>0.1){
		cat("The LM test fails to reject the NULL hypothesis of a restricted model at the 10% significance level\n")
	} else{
		cat("The LM test rejects the NULL hypothesis of a restricted model at the 10% significance level\n")
	}
	cat("\nLR Test (Restricted versus Unrestricted Model)\n\n")
	lr.stat = 2*(likelihood(fit.gh) - likelihood(fit.nig))
	lr.pvalue = 1 - pchisq(lr.stat, 1)
	print(data.frame(LR.stat = lr.stat, dof = 1, p.value = lr.pvalue))
	if(lmtest.pvalue>0.1){
		cat("The LR test fails to reject the NULL hypothesis of a restricted model at the 10% significance level\n")
	} else{
		cat("The LR test rejects the NULL hypothesis of a restricted model at the 10% significance level\n")
	}
	
	sc = apply(fit.hyp@fit$scores, 2, "sum")
	lmtest.v = (t(sc)%*%solve(t(fit.hyp@fit$scores)%*%fit.hyp@fit$scores)%*%(sc))
	# equivalent to summary(lm(ones~scores-1))$r.squared*N
	lmtest.pvalue = 1 - pchisq(lmtest.v, 1)
	cat("\nNull Hypothesis of HYP distribution (lambda = 1 in GHYP family):\n")
	cat("\n\n")
	cat("\nLM Test (Restriction: lambda = 1)\n\n")
	print(data.frame(LM.stat = lmtest.v, dof = 1, p.value = lmtest.pvalue))
	if(lmtest.pvalue>0.1){
		cat("The LM test fails to reject the NULL hypothesis of a restricted model at the 10% significance level\n")
	} else{
		cat("The LM test rejects the NULL hypothesis of a restricted model at the 10% significance level\n")
	}
	cat("\nLR Test (Restricted versus Unrestricted Model)\n\n")
	lr.stat = 2*(likelihood(fit.gh) - likelihood(fit.hyp))
	lr.pvalue = 1 - pchisq(lr.stat, 1)
	print(data.frame(LR.stat = lr.stat, dof = 1, p.value = lr.pvalue))
	if(lmtest.pvalue>0.1){
		cat("The LR test fails to reject the NULL hypothesis of a restricted model at the 10% significance level\n")
	} else{
		cat("The LR test rejects the NULL hypothesis of a restricted model at the 10% significance level\n")
	}
	sink(type="message")
	sink()
	close(zz)
	
	postscript("test10-2.eps", width = 12, height = 12)
	par(mfrow = c(2, 1))
	z = fit.hyp@fit$z
	result 	= hist(x = z, col = "grey", border = "white",
			breaks = "FD", main = 
					paste("Empirical Density of Standardized Residuals",sep=""),
			xlim = c(min(z)*1.2, 1.2*max(z)),ylim=c(0,0.6), probability = TRUE, ylab="Probability", 
			cex.main = 0.8)
	s = result$breaks
	box()
	distribution = fit.gh@model$distribution
	fd 	= .getDensity(distribution)
	Names 	= fit.gh@model$modelnames
	if(any(substr(Names, 1, 7)=="dlambda")) dlambda = fit.gh@fit$coef["dlambda"] else dlambda = 0
	if(any(substr(Names, 1, 5)=="shape")) shape = fit.gh@fit$coef["shape"] else shape = 4
	if(any(substr(Names, 1, 4)=="skew")) skew = fit.gh@fit$coef["skew"] else skew = 4
	y = fd(z = s, hh = 1, lambda = dlambda, skew = skew, shape = shape)
	lines(s, y, lwd = 2, col = 2)
	
	distribution = fit.nig@model$distribution
	fd 	= .getDensity(distribution)
	Names = fit.nig@model$modelnames
	if(any(substr(Names, 1, 7)=="dlambda")) dlambda = fit.nig@fit$coef["dlambda"] else dlambda = 0
	if(any(substr(Names, 1, 5)=="shape")) shape = fit.nig@fit$coef["shape"] else shape = 4
	if(any(substr(Names, 1, 4)=="skew")) skew = fit.nig@fit$coef["skew"] else skew = 4
	y	= fd(z = s, hh = 1, lambda = dlambda, skew = skew, shape = shape)
	lines(s, y, lwd = 2, col = 3)
	
	distribution = fit.hyp@model$distribution
	fd 	= .getDensity(distribution)
	Names 	= fit.hyp@model$modelnames
	if(any(substr(Names, 1, 7)=="dlambda")) dlambda = fit.hyp@fit$coef["dlambda"] else dlambda = 0
	if(any(substr(Names, 1, 5)=="shape")) shape = fit.hyp@fit$coef["shape"] else shape = 4
	if(any(substr(Names, 1, 4)=="skew")) skew = fit.hyp@fit$coef["skew"] else skew = 4
	y	= fd(z = s, hh = 1, lambda = dlambda, skew = skew, shape = shape)
	lines(s, y, lwd = 2, col = 4)
	legend("topleft", legend = c("GHYP", "NIG", "HYP"), col = 2:4, fill = 2:4, bty = "n")
	
	####
	distribution = fit.gh@model$distribution
	qDist.gh = .getQuantile(distribution)
	Names 	= fit.gh@model$modelnames
	if(any(substr(Names, 1, 7)=="dlambda")) dlambda.gh = fit.gh@fit$coef["dlambda"] else dlambda.gh = 0
	if(any(substr(Names, 1, 5)=="shape")) shape.gh = fit.gh@fit$coef["shape"] else shape.gh = 4
	if(any(substr(Names, 1, 4)=="skew")) skew.gh = fit.gh@fit$coef["skew"] else skew.gh = 4
	z1 = qDist.gh(ppoints(length(z)), lambda = dlambda.gh, skew = skew.gh, shape = shape.gh)[order(order(z))]
	
	distribution = fit.nig@model$distribution
	qDist.nig = .getQuantile(distribution)
	Names 	= fit.nig@model$modelnames
	if(any(substr(Names, 1, 7)=="dlambda")) dlambda.nig = fit.nig@fit$coef["dlambda"] else dlambda.nig = 0
	if(any(substr(Names, 1, 5)=="shape")) shape.nig = fit.nig@fit$coef["shape"] else shape.nig = 4
	if(any(substr(Names, 1, 4)=="skew")) skew.nig = fit.nig@fit$coef["skew"] else skew.nig = 4
	z2 = qDist.nig(ppoints(length(z)), lambda = dlambda.nig, skew = skew.nig, shape = shape.nig)[order(order(z))]
	
	distribution = fit.hyp@model$distribution
	qDist.hyp = .getQuantile(distribution)
	Names 	= fit.hyp@model$modelnames
	if(any(substr(Names, 1, 7)=="dlambda")) dlambda.hyp = fit.hyp@fit$coef["dlambda"] else dlambda.hyp = 0
	if(any(substr(Names, 1, 5)=="shape")) shape.hyp = fit.hyp@fit$coef["shape"] else shape.hyp = 4
	if(any(substr(Names, 1, 4)=="skew")) skew.hyp = fit.hyp@fit$coef["skew"] else skew.hyp = 4
	z3 = qDist.hyp(ppoints(length(z)), lambda = dlambda.hyp, skew = skew.hyp, shape = shape.hyp)[order(order(z))]
	
	ylim = range(z)
	plot(z1, z, main = " GHYP - QQ Plot",
			xlab = "Theoretical Quantiles", ylab = "Sample Quantiles",
			col = 1, cex = 0.7, ylim = ylim)
	points(z2, z, col = 2, pch = 2)
	points(z3, z, col = 3, pch = 3)
	.qqLine(y = z, dist = "ghyp", datax = FALSE, lambda = dlambda.gh, skew = skew.gh, shape = shape.gh)
	legend("topleft", legend = c("GHYP", "NIG", "HYP"), col = 1:3,  bty = "n",
			pch = 1:3)
	grid()
	dev.off()
	
}