#################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009, 2010
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################

#################################################################################
# DCC model
#################################################################################

#  setwd("../inst/rgarch.tests/Results")
#--------------------------------------------------------------------------------
# Fit & Filter Test

# Model1 : AR(1) - APARCH(1,1) - DCC(1,1) Model
# Details: Common variance model and mean model lag specification

rgarch.test121 = function()
{
	data(dji30ret)
	Dat = dji30ret[, 1:10, drop = FALSE]
	
	uspec = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "apARCH"), 
			distribution.model = "norm")
	
	spec = dccspec(uspec = multispec( replicate(10, uspec) ), dccOrder = c(1,1), distribution = "mvnorm")
	
	# fit the model
	fit = dccfit(data = Dat, spec = spec, out.sample = 100, solver = "solnp", 
			fit.control = list(scale = TRUE, eval.se = TRUE, use.multicore = FALSE))
	
	# show how to filter:
	# duplicate the spec
	spec2 = spec
	# fix the dcc parameters
	spec2@mspec$optimization.model$fixed.pars = coef(fit, type = "dcc")
	# fix the garchpars
	for(i in 1:10) spec2@uspec@spec[[i]]@optimization.model$fixed.pars = coef(fit@ufit@fit[[i]])
	filt = dccfilter(spec = spec2, data = Dat, out.sample = 100)
	
	
	options(width = 120)
	zz <- file("test12-1.txt", open="wt")
	sink(zz)
	# compare likelihood of fitted and filtered
	cat("\nFit and Filtered DCC models LLH check:\n")
	print( data.frame( fit = likelihood(fit), filter = likelihood(filt), row.names = "LLH") )
	# compare first and last conditional correlation matrices
	cat("\nFit and Filtered DCC models Conditional Correlation check:\n")
	cat("\nFirst:\n")
	print( rcov(fit)[,,1] == rcov(filt)[,,1] )
	cat("\nLast:\n")
	print( last(rcov(fit)) == last(rcov(filt)) )
	# Fit Summary and Plots
	(fit)
	sink(type="message")
	sink()
	close(zz)
	
	# Evaluate and Save the plots
	postscript("test12-1a.eps", width = 12, height = 12)
	plot(fit, series = 1:10, which = 1)
	dev.off()
	postscript("test12-1b.eps", width = 12, height = 12)
	plot(fit, series = 1:10, which = 2)
	dev.off()
	postscript("test12-1c.eps", width = 12, height = 12)
	plot(fit, series = 1:5, which = 3)
	dev.off()
	postscript("test12-1d.eps", width = 12, height = 12)
	plot(fit, series = 1:5, which = 4)
	dev.off()
	postscript("test12-1e.eps", width = 12, height = 12)
	plot(fit, which = 5)
	dev.off()
	
}

# Model2 : 
# Details: Different models combined in the univariate specification
rgarch.test122 = function()
{
	data(dji30ret)
	Dat = dji30ret[, 1:4, drop = FALSE]
	uspec1 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "apARCH"), 
			distribution.model = "norm")
	uspec2 = ugarchspec(mean.model = list(armaOrder = c(2,0)), variance.model = list(model = "gjrGARCH"), 
			distribution.model = "norm")
	uspec3 = ugarchspec(mean.model = list(armaOrder = c(0,0)), variance.model = list(model = "sGARCH"), 
			distribution.model = "norm")
	uspec4 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "sGARCH",
					garchOrder = c(2, 1)), 
			distribution.model = "norm")
	uspec = c( uspec1, uspec2, uspec3, uspec4 )
	spec = dccspec(uspec = multispec( uspec ), dccOrder = c(1,1), distribution = "mvlaplace")
	
	# fit the model
	fit = dccfit(data = Dat, spec = spec, out.sample = 100, solver = "solnp", 
			fit.control = list(scale = TRUE, eval.se = TRUE, use.multicore = FALSE))
	
	# check that the filter can also deal with non-equal spec
	spec2 = spec
	# fix the dcc parameters
	spec2@mspec$optimization.model$fixed.pars = coef(fit, type = "dcc")
	# fix the garchpars
	for(i in 1:4) spec2@uspec@spec[[i]]@optimization.model$fixed.pars = coef(fit@ufit@fit[[i]])
	filt = dccfilter(spec = spec2, data = Dat, out.sample = 100)
	
	
	options(width = 120)
	zz <- file("test12-2.txt", open="wt")
	sink(zz)
	# compare likelihood of fitted and filtered
	cat("\nFit and Filtered DCC models LLH check:\n")
	print( data.frame( fit = likelihood(fit), filter = likelihood(filt), row.names = "LLH") )
	# compare first and last conditional correlation matrices
	cat("\nFit and Filtered DCC models Conditional Correlation check:\n")
	cat("\nFirst:\n")
	print( rcov(fit)[,,1] == rcov(filt)[,,1] )
	cat("\nLast:\n")
	print( last(rcov(fit)) == last(rcov(filt)) )
	# Fit Summary and Plots
	(fit)
	sink(type="message")
	sink()
	close(zz)
	
	# Evaluate and Save the plots
	postscript("test12-2a.eps", width = 12, height = 12)
	plot(fit, series = 1:4, which = 1)
	dev.off()
	postscript("test12-2b.eps", width = 12, height = 12)
	plot(fit, series = 1:4, which = 2)
	dev.off()
	postscript("test12-2c.eps", width = 12, height = 12)
	plot(fit, series = 1:4, which = 3)
	dev.off()
	postscript("test12-2d.eps", width = 12, height = 12)
	plot(fit, series = 1:4, which = 4)
	dev.off()
	postscript("test12-2e.eps", width = 12, height = 12)
	plot(fit, which = 5)
	dev.off()
}

#--------------------------------------------------------------------------------
# Forecast Test
# Details: Different models combined in the univariate specification
rgarch.test123 = function()
{
	data(dji30ret)
	Dat = dji30ret[, 1:4, drop = FALSE]
	uspec1 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "apARCH"), 
			distribution.model = "norm")
	uspec2 = ugarchspec(mean.model = list(armaOrder = c(2,0)), variance.model = list(model = "gjrGARCH"), 
			distribution.model = "norm")
	uspec3 = ugarchspec(mean.model = list(armaOrder = c(0,0)), variance.model = list(model = "sGARCH"), 
			distribution.model = "norm")
	uspec4 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "sGARCH",
					garchOrder = c(2, 1)), 
			distribution.model = "norm")
	uspec = c( uspec1, uspec2, uspec3, uspec4 )
	spec = dccspec(uspec = multispec( uspec ), dccOrder = c(1,1), distribution = "mvt")
	
	# fit the model
	fit = dccfit(data = Dat, spec = spec, out.sample = 100, solver = "solnp", 
			fit.control = list(scale = TRUE, eval.se = FALSE, use.multicore = FALSE))
	
	# rolling forecast
	forc1 = dccforecast(fit, n.ahead = 1, n.roll = 99)
	# unconditional forecast
	forc2 = dccforecast(fit, n.ahead = 100, n.roll = 0)
	# combination of the 2
	forc3 = dccforecast(fit, n.ahead = 2, n.roll = 99)
	
	H1 = rcov(forc1)
	H3 = rcov(forc3)
	
	options(width = 120)
	zz <- file("test12-3.txt", open="wt")
	sink(zz)
	cat("\nDCC Forecast Summary:\n")
	(forc1)
	(forc2)
	(forc3)
	cat("\nRolling and mixed Rolling Forecast DCC model Conditional Covariance check:\n")
	cat("\nFirst:\n")
	print( H1[[1]][,,1] == H3[[1]][,,1] )
	cat("\nLast:\n")
	print( H1[[100]][,,1] == H3[[100]][,,1] )
	sink(type="message")
	sink()
	close(zz)
	
	# Evaluate and Save the plots
	postscript("test12-3a.eps", width = 12, height = 12)
	plot(forc1, series = 1:4, which = 1)
	dev.off()
	postscript("test12-3b.eps", width = 12, height = 12)
	plot(forc1, series = 1:4, which = 2)
	dev.off()
	postscript("test12-3c.eps", width = 12, height = 12)
	plot(forc1, series = 1:4, which = 3)
	dev.off()
	postscript("test12-3d.eps", width = 12, height = 12)
	plot(forc1, series = 1:4, which = 4)
	dev.off()
	postscript("test12-3e.eps", width = 12, height = 12)
	par(mfrow = c(2,1))
	plot(forc1, which = 5)
	plot(forc2, which = 5)
	dev.off()
}
#--------------------------------------------------------------------------------
# Simulation Test
rgarch.test124 = function()
{
	data(dji30ret)
	Dat = dji30ret[, 1:4, drop = FALSE]
	uspec1 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "apARCH"), 
			distribution.model = "norm")
	uspec2 = ugarchspec(mean.model = list(armaOrder = c(2,0)), variance.model = list(model = "gjrGARCH"), 
			distribution.model = "norm")
	uspec3 = ugarchspec(mean.model = list(armaOrder = c(0,0)), variance.model = list(model = "sGARCH"), 
			distribution.model = "norm")
	uspec4 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "sGARCH",
					garchOrder = c(2, 1)), 
			distribution.model = "norm")
	uspec = c( uspec1, uspec2, uspec3, uspec4 )
	spec = dccspec(uspec = multispec( uspec ), dccOrder = c(1,1), distribution = "mvt")
	
	# fit the model
	fit = dccfit(data = Dat, spec = spec, out.sample = 100, solver = "solnp", 
			fit.control = list(scale = TRUE, eval.se = FALSE, use.multicore = FALSE))
	
	# simulation takes either fit or spec (check both)
	spec2 = spec
	# fix the dcc parameters
	spec2@mspec$optimization.model$fixed.pars = coef(fit, type = "dcc")
	# fix the garchpars
	for(i in 1:4) spec2@uspec@spec[[i]]@optimization.model$fixed.pars = coef(fit@ufit@fit[[i]])
	
	preR = cor(as.matrix(Dat))
	presigma = tail( sigma(fit@ufit), 2 )
	preresiduals = tail( residuals(fit@ufit), 2 )
	prereturns = tail( as.matrix(Dat), 2 )
	# also possible to obtain uncvariance by supplying parameters
	# showMethods("uncvariance")
	uncv = sapply(fit@ufit@fit, FUN = function(x) uncvariance(x))
	preH = diag(sqrt(uncv)) %*% preR %*% diag(sqrt(uncv))
			
	sim1 = dccsim(fitORspec = fit, n.sim = 1000, n.start = 100, m.sim = 2, startMethod = "sample", 
		presigma = presigma, preresiduals = preresiduals, prereturns = prereturns, preR = preR, preH = NULL, 
		rseed = c(100, 200), mexsimdata = NULL, vexsimdata = NULL)
	

	sim2 = dccsim(fitORspec = spec2, n.sim = 1000, n.start = 100, m.sim = 2, 
		presigma = presigma, preresiduals = preresiduals, prereturns = prereturns, preR = preR, preH = preH, 
		rseed = c(100, 200), mexsimdata = NULL, vexsimdata = NULL)


	R1 = rcor(sim1, sim = 1)
	R2 = rcor(sim2, sim = 1)
	H1 = rcov(sim1, sim = 2)
	H2 = rcov(sim2, sim = 2)
	
	
	# now replicate the conditions for the sample start
	sim3 = dccsim(fitORspec = fit, n.sim = 1000, n.start = 100, m.sim = 2, startMethod = "sample", 
			presigma = NULL, preresiduals = NULL, prereturns = NULL, preR = preR, preH = NULL, 
			rseed = c(100, 200), mexsimdata = NULL, vexsimdata = NULL)
	
	# prereturns should be NULL
	sim4 = dccsim(fitORspec = spec2, n.sim = 1000, n.start = 100, m.sim = 2, 
			presigma = presigma, preresiduals = preresiduals, prereturns = NULL, preR = preR, preH = preH, 
			rseed = c(100, 200), mexsimdata = NULL, vexsimdata = NULL)
	
	R3 = rcor(sim3, sim = 2)
	R4 = rcor(sim4, sim = 2)
	H3 = rcor(sim3, sim = 1)
	H4 = rcor(sim4, sim = 1)
	
	
	options(width = 120)
	zz <- file("test12-4.txt", open="wt")
	sink(zz)
	cat("\nDCC Simulation Tests:\n")
	cat("\nFit and Spec based Simulation DCC model Conditional Correlation check1:\n")
	cat("\nn = 1\n")
	print(R1[, , 1] == R2[, , 1])
	cat("\nn = 100\n")
	print(R1[, , 100] == R2[, , 100])
	cat("\nFit and Spec based Simulation DCC model Conditional Covariance check1:\n")
	cat("\nn = 1\n")
	print(H1[, , 1] == H2[, , 1])
	cat("\nn = 100\n")
	print(H1[, , 100] == H2[, , 100])
	cat("\nFit and Spec based Simulation DCC model Conditional Correlation check2:\n")
	cat("\nn = 1\n")
	print(R3[, , 1] == R4[, , 1])
	cat("\nn = 100\n")
	print(R3[, , 100] == R4[, , 100])
	cat("\nFit and Spec based Simulation DCC model Conditional Covariance check2:\n")
	cat("\nn = 1\n")
	print(H3[, , 1] == H4[, , 1])
	cat("\nn = 100\n")
	print(H3[, , 100] == H4[, , 100])
	sink(type="message")
	sink()
	close(zz)
}
#--------------------------------------------------------------------------------
# Roll Test
rgarch.test125 = function()
{
	
	Dat = dji30ret[, 1:5, drop = FALSE]
	uspec1 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "apARCH"), 
			distribution.model = "norm")
	uspec2 = ugarchspec(mean.model = list(armaOrder = c(2,0)), variance.model = list(model = "gjrGARCH"), 
			distribution.model = "norm")
	uspec3 = ugarchspec(mean.model = list(armaOrder = c(0,0)), variance.model = list(model = "sGARCH"), 
			distribution.model = "norm")
	uspec4 = ugarchspec(mean.model = list(armaOrder = c(1,0)), variance.model = list(model = "sGARCH",
					garchOrder = c(2, 1)), 
			distribution.model = "norm")
	uspec5 = ugarchspec(mean.model = list(armaOrder = c(1,1)), variance.model = list(model = "eGARCH",
					garchOrder = c(1, 1)), 
			distribution.model = "norm")
	
	uspec = c( uspec1, uspec2, uspec3, uspec4, uspec5 )
	spec = dccspec(uspec = multispec( uspec ), dccOrder = c(1,1), distribution = "mvt")
	
	roll = dccroll(spec, data = Dat, n.ahead = 1, forecast.length = 100, refit.every = 25, 
			refit.window = "moving", solver = "solnp", 
			fit.control = list(eval.se = FALSE, use.multicore = FALSE), solver.control = list(), 
			save.fit = FALSE, save.wdir = NULL, trace = TRUE)
	
	options(width = 120)
	zz <- file("test12-5-1.txt", open="wt")
	sink(zz)
	roll
	cat("\nAverage Log-Likelihood Across Rolls:\n")
	print( round( likelihood(object)/object@spec$n.data, 2) )
	sink(type="message")
	sink()
	close(zz)
	
	options(width = 120)
	zz <- file("test12-5-2.txt", open="wt")
	sink(zz)
	cat("\nCorrelation from roll object\n")
	cat("\nis(rcor(roll, roll = 1))[1]\n")
	is(rcor(roll))[1]
	cat("\nThere are 4 rolls (forecast.length/refit.every).")
	cat("\nEach rolling window correlation is a list of arrays of Length = refit.every,")
	cat("\nwith each array equal to m x m (no. assets) by n.ahead.\n")
	cat("\nSince n.ahead = 1, reshape list to array:.")
	cat("\narray(unlist(rcor(roll, roll = 1)) , dim = c(5,5,25))\n")
	array(unlist(rcor(roll)) , dim = c(5,5,25))
	sink(type="message")
	sink()
	close(zz)
	
	postscript("test12-5.eps", width = 12, height = 12)
	par(mfrow = c(2, 2))
	for(i in 1:4) plot(roll, which = 5, roll = i)
	dev.off()
	
}