#################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009, 2010
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################
# setwd("../inst/rgarch.tests/")
#################################################################################
# ARFIMAX model
#################################################################################

# fitting
rgarch.test131 = function()
{
	data(sp500ret)
	Dat = sp500ret[, 1, drop = FALSE]
	
	fit = vector(mode = "list", length = 9)
	dist = c("norm", "snorm", "std", "sstd", "ged", "sged", "nig", "ghyp", "jsu")
	for(i in 1:9){
		spec = arfimaspec( mean.model = list(armaOrder = c(1,1), include.mean = TRUE, arfima = FALSE), 
				distribution.model = dist[i])
		fit[[i]] = arfimafit(spec = spec, data = Dat, solver = "solnp", fit.control = list(scale = 1))
	}
	cfmatrix = matrix(NA, nrow = 9, ncol = 7)
	colnames(cfmatrix) = c("mu", "ar1", "ma1", "sigma", "skew", "shape", "dlambda")
	rownames(cfmatrix) = dist
	
	for(i in 1:9){
		cf = coef(fit[[i]])
		cfmatrix[i, match(names(cf), colnames(cfmatrix))] =  cf
	}
	sk = ku = rep(0, 9)
	for(i in 1:9){
		cf = coef(fit[[i]])
		if(fit[[i]]@model$include.skew) sk[i] = dskewness(distribution = dist[i], 
					skew = cf["skew"], shape = cf["shape"], lambda = cf["dlambda"])		
		if(fit[[i]]@model$include.shape) ku[i] = dkurtosis(distribution = dist[i], 
					skew = cf["skew"], shape = cf["shape"], lambda = cf["dlambda"])
	}
	hq = sapply(fit, FUN = function(x) infocriteria(x)[4])
	cfmatrix = cbind(cfmatrix, sk, ku, hq)
	colnames(cfmatrix) = c(colnames(cfmatrix[,1:7]), "skewness", "ex.kurtosis","HQIC")
	
	options(width = 120)
	zz <- file("test13-1-1.txt", open="wt")
	sink(zz)
	print(cfmatrix, digits = 4)
	sink(type="message")
	sink()
	close(zz)
	
	# filter the data to check results
	filt = vector(mode = "list", length = 9)
	for(i in 1:9){
		spec = arfimaspec( mean.model = list(armaOrder = c(1,1), include.mean = TRUE, arfima = FALSE), 
				distribution.model = dist[i])
		spec@optimization.model$fixed.pars = as.list(coef(fit[[i]]))
		filt[[i]] = arfimafilter(spec = spec, data = Dat)
	}
	options(width = 120)
	zz <- file("test13-1-2.txt", open="wt")
	sink(zz)
	cat("\nARFIMAfit and ARFIMAfilter residuals check:")
	cat("\nFiltered\n")
	print(head(sapply(filt, FUN = function(x) residuals(x))))
	cat("\nFitted\n")	
	print(head(sapply(fit, FUN = function(x) residuals(x))))
	sink(type="message")
	sink()
	close(zz)	
}

# unconditional forecasting
rgarch.test132 = function()
{
	data(sp500ret)
	Dat = sp500ret[, 1, drop = FALSE]
	
	fit = vector(mode = "list", length = 9)
	dist = c("norm", "snorm", "std", "sstd", "ged", "sged", "nig", "ghyp", "jsu")
	for(i in 1:9){
		spec = arfimaspec( mean.model = list(armaOrder = c(1,1), include.mean = TRUE, arfima = FALSE), 
				distribution.model = dist[i])
		fit[[i]] = arfimafit(spec = spec, data = Dat, solver = "solnp", fit.control = list(scale = 1))
	}
	cfmatrix = matrix(NA, nrow = 9, ncol = 7)
	colnames(cfmatrix) = c("mu", "ar1", "ma1", "sigma", "skew", "shape", "dlambda")
	rownames(cfmatrix) = dist
	
	for(i in 1:9){
		cf = coef(fit[[i]])
		cfmatrix[i, match(names(cf), colnames(cfmatrix))] =  cf
	}
	
	umean = rep(0, 9)
	for(i in 1:9){
		umean[i] = uncmean(fit[[i]])
	}
	
	forc = vector(mode = "list", length = 9)
	for(i in 1:9){
		forc[[i]] = arfimaforecast(fit[[i]], n.ahead = 100)
	}
	
	lmean40 = sapply(forc, FUN = function(x) as.numeric(as.data.frame(x)[40,1]))
	cfmatrix1 = cbind(cfmatrix, umean, lmean40)
	colnames(cfmatrix1) = c(colnames(cfmatrix1[,1:7]), "uncmean", "forecast40")
	
	# forecast with spec to check results
	forc2 = vector(mode = "list", length = 9)
	for(i in 1:9){
		spec = arfimaspec( mean.model = list(armaOrder = c(1,1), include.mean = TRUE, arfima = FALSE), 
				distribution.model = dist[i])
		spec@optimization.model$fixed.pars = as.list(coef(fit[[i]]))
		forc2[[i]] = arfimaforecast(spec, data = Dat, n.ahead = 100)
	}
	lmean240 = sapply(forc2, FUN = function(x) as.numeric(as.data.frame(x)[40,1]))
	cfmatrix2 = cbind(cfmatrix, umean, lmean240)
	colnames(cfmatrix2) = c(colnames(cfmatrix2[,1:7]), "uncmean", "forecast40")
	
	
	options(width = 120)
	zz <- file("test13-2-1.txt", open="wt")
	sink(zz)
	cat("\nARFIMAforecast from ARFIMAfit and ARFIMAspec check:")
	cat("\nFit\n")	
	print(cfmatrix1, digits = 4)
	cat("\nSpec\n")	
	print(cfmatrix2, digits = 4)
	sink(type="message")
	sink()
	close(zz)
	
	
	
	nforc = sapply(forc, FUN = function(x) t(unlist(as.data.frame(x, aligned = FALSE))))
	
	postscript("test13-2.eps", width = 12, height = 5)
	dd = c(rownames(tail(Dat, 500)), rownames(as.data.frame(forc[[1]])))
	clrs = rainbow(9, alpha = 1, start = 0.4, end = 0.95)
	plot(as.Date(dd), c(tail(Dat[,1], 500), nforc[,1]), type = "l", ylim = c(-0.02, 0.02), col = "lightgrey",
			ylab = "", xlab = "", main = "100-ahead Unconditional Forecasts")
	for(i in 1:9){
		tmp = c(tail(Dat[,1], 500), rep(NA, 100))
		tmp[501:600] = nforc[1:100,i]
		lines(as.Date(dd), c(rep(NA, 500), tmp[-(1:500)]), col = clrs[i])
	}
	legend("topleft", legend = dist, col = clrs, fill = clrs, bty = "n")
	dev.off()
	
}


# rolling forecast
rgarch.test133 = function()
{
	data(sp500ret)
	Dat = sp500ret[, 1, drop = FALSE]
	
	fit = vector(mode = "list", length = 9)
	dist = c("norm", "snorm", "std", "sstd", "ged", "sged", "nig", "ghyp", "jsu")
	for(i in 1:9){
		spec = arfimaspec( mean.model = list(armaOrder = c(1,1), include.mean = TRUE, arfima = FALSE), 
				distribution.model = dist[i])
		fit[[i]] = arfimafit(spec = spec, data = Dat, solver = "solnp", 
				out.sample = 1000, fit.control = list(scale = 1))
	}
	cfmatrix = matrix(NA, nrow = 9, ncol = 7)
	colnames(cfmatrix) = c("mu", "ar1", "ma1", "sigma", "skew", "shape", "dlambda")
	rownames(cfmatrix) = dist
	
	for(i in 1:9){
		cf = coef(fit[[i]])
		cfmatrix[i, match(names(cf), colnames(cfmatrix))] =  cf
	}
	
	
	forc = vector(mode = "list", length = 9)
	for(i in 1:9){
		forc[[i]] = arfimaforecast(fit[[i]], n.ahead = 1, n.roll = 999)
	}
	rollforc = sapply(forc, FUN = function(x) t(unlist(as.data.frame(x, rollframe = "all", aligned = FALSE))))
	

	
	# forecast performance measures:
	fpmlist = vector(mode = "list", length = 9)
	for(i in 1:9){
		fpmlist[[i]] = fpm(forc[[i]], type = 2)
	}
	
	postscript("test13-3.eps", width = 12, height = 5)
	par(mfrow = c(1,2))
	dd = rownames(tail(Dat, 1250))
	clrs = rainbow(9, alpha = 1, start = 0.4, end = 0.95)
	plot(as.Date(dd), tail(Dat[,1], 1250), type = "l", ylim = c(-0.02, 0.02), col = "lightgrey",
			ylab = "", xlab = "", main = "Rolling 1-ahead Forecasts\nvs Actual")
	for(i in 1:9){
		tmp = tail(Dat[,1], 1250)
		tmp[251:1250] = rollforc[1:1000,i]
		lines(as.Date(dd), c(rep(NA, 250), tmp[-(1:250)]), col = clrs[i])
	}
	legend("topleft", legend = dist, col = clrs, fill = clrs, bty = "n")
	
	# plot deviation measures and range
	tmp = vector(mode = "list", length = 9)
	for(i in 1:9){
		tmp[[i]] = fpmlist[[i]]@seriesfpm$lossfn$mad
		names(tmp[[i]]) = dist[i]
	}
	boxplot(tmp, col = clrs, names = dist, range  = 6, notch = TRUE, 
			main = "Rolling 1-ahead Forecasts\nAbsolute Deviation Loss")
	dev.off()
	
	# fpm comparison
	compm = matrix(NA, nrow = 3, ncol = 9)
	rownames(compm) = c("MSE", "MAD", "DAC")
	compm = sapply(fpmlist, FUN = function(x) c(x@seriesfpm$meanloss["mse"], x@seriesfpm$meanloss["mad"],
								x@seriesfpm$meanloss["dac"]))
	colnames(compm) = dist
			
	zz <- file("test13-3.txt", open="wt")
	sink(zz)
	cat("\nRolling Forecast FPM\n")
	print(compm, digits = 4)
	sink(type="message")
	sink()
	close(zz)
	
}

# multi-methods
rgarch.test134 = function()
{
	data(dji30ret)
	Dat = dji30ret[, 1:3, drop = FALSE]
	
	spec1 = arfimaspec(mean.model = list(armaOrder = c(2,1)))
	spec2 = arfimaspec(mean.model = list(armaOrder = c(2,2)))
	spec3 = arfimaspec(mean.model = list(armaOrder = c(1,1)), distribution.model = "sstd")
	speclist = as.list(c(spec1, spec2, spec3))
	mspec = multispec( speclist )
	
	mfit1 = multifit(multispec = mspec, data = Dat)
	
	spec1 = arfimaspec(mean.model = list(armaOrder = c(1,1)))
	mspec = multispec( replicate(3, spec1) )
	
	mfit2 = multifit(multispec = mspec, data = Dat)
	
	zz <- file("test13-4-1.txt", open="wt")
	sink(zz)
	cat("\nMultifit Evaluation\n")
	cat("\nUnequal Spec\n")
	mfit1
	cat("\nEqual Spec\n")
	mfit2
	sink(type="message")
	sink()
	close(zz)
	
	
	mforc1 = multiforecast(mfit1, n.ahead = 10)
	mforc2 = multiforecast(mfit2, n.ahead = 10)
	
	zz <- file("test13-4-2.txt", open="wt")
	sink(zz)
	cat("\nMultiforecast Evaluation\n")
	cat("\nUnequal Spec\n")
	mforc1
	cat("\nEqual Spec\n")
	mforc2
	sink(type="message")
	sink()
	close(zz)
		
}


# rolling estimation
rgarch.test135 = function()
{
	data(sp500ret)
	Dat = sp500ret[, 1, drop = FALSE]
	spec = arfimaspec()
	roll = arfimaroll(spec,  data = Dat, n.ahead = 1, forecast.length = 500, 
			refit.every = 25, refit.window = "moving", use.multicore = FALSE, 
			solver = "solnp", fit.control = list(), solver.control = list() ,
			calculate.VaR = TRUE, VaR.alpha = c(0.01, 0.05))
	
	zz <- file("test13-5.txt", open="wt")
	sink(zz)
	cat("\nRolling Forecast Evaluation\n")
	report(roll, "VaR")
	report(roll, "fpm")
	sink(type="message")
	sink()
	close(zz)
}


# parameter distribution
rgarch.test136 = function()
{
	
	Dat = sp500ret[, 1, drop = FALSE]	
	dist = "norm"
	#min(Mod(polyroot(c(1, -c(0.6, 0.1)))))
	spec = arfimaspec( mean.model = list(armaOrder = c(2,2), include.mean = TRUE, arfima = FALSE), 
				distribution.model = dist, fixed.pars = list(mu = 0.02, ar1 = 0.6, ar2 = 0.1, ma1 = -0.8,
		ma2 = -0.1, sigma = 0.0123))
		
	pdist = arfimadistribution(spec, data = NULL, n.sim = 2000, n.start = 100, 
			m.sim = 100, recursive = TRUE, recursive.length = 10000, recursive.window = 1000)
	
	
	zz <- file("test13-6-1.txt", open="wt")
	sink(zz)
	cat("\nARFIMA model parameter distribution\n")
	pdist
	sink(type="message")
	sink()
	close(zz)

	
}

rgarch.test137 = function()
{
	# a test of equivalence between arima.sim, arfima and arma simulations
	
	# First arfima and arma
	spec1 = arfimaspec( mean.model = list(armaOrder = c(2,1), include.mean = TRUE, arfima = TRUE), 
			distribution.model = "std", fixed.pars = list(mu = 0.02, ar1 = 0.6, ar2 = 0.01, ma1 = -0.7, darfima = 0,
					shape = 5, sigma = 0.0123))
	
	spec2 = arfimaspec( mean.model = list(armaOrder = c(2,1), include.mean = TRUE, arfima = FALSE), 
			distribution.model = "std", fixed.pars = list(mu = 0.02, ar1 = 0.6, ar2 = 0.01, ma1 = -0.7,
					shape = 5,sigma = 0.0123))
	
	sim1 = arfimapath(spec1, n.sim = 1000, m.sim = 1, rseed = 100)
	sim2 = arfimapath(spec2, n.sim = 1000, m.sim = 1, rseed = 100)
	
	zz <- file("test13-7-1.txt", open="wt")
	sink(zz)
	cat("\nARFIMA and ARMA simulation tests:\n")
	print(head(as.data.frame(sim1)), digits = 5)
	print(head(as.data.frame(sim2)), digits = 5)
	sink(type="message")
	sink()
	close(zz)

	# Now the rgarch simulation of ARFIMA/ARMA with arima.sim of R
	# Note that arima.sim simulates the residuals (i.e no mean):
	
	#  First we simulate common innovations:
	set.seed(100)
	inn = rdist("std", 1000, mu = 0, sigma = 1, lambda = 0, skew = 0, shape = 5)
	
	spec1 = arfimaspec( mean.model = list(armaOrder = c(2,1), include.mean = FALSE, arfima = TRUE), 
			distribution.model = "std", fixed.pars = list(ar1 = 0.6, ar2 = 0.01, ma1 = -0.7, darfima = 0,
					shape = 5, sigma = 0.0123))
	spec2 = arfimaspec( mean.model = list(armaOrder = c(2,1), include.mean = FALSE, arfima = FALSE), 
			distribution.model = "std", fixed.pars = list(ar1 = 0.6, ar2 = 0.01, ma1 = -0.7,
					shape = 5,sigma = 0.0123))
	
	sim1 = arfimapath(spec1, n.sim = 1000, m.sim = 1, rseed = 100, preres = c(0,0), prereturns = c(0,0), 
			custom.dist = list(name = "sample",
					distfit = matrix(inn, ncol = 1)))
	sim2 = arfimapath(spec2, n.sim = 1000, m.sim = 1, rseed = 100, preres = c(0,0), prereturns = c(0,0), 
			custom.dist = list(name = "sample",
					distfit = matrix(inn, ncol = 1)))

	# Note that we pass the non-standardized innovations to arima.sim (i.e. multiply by sigma)
	sim3 = arima.sim(model = list(ar = c(0.6, 0.01), ma = -0.7), n = 1000,
			n.start = 3, start.innov = c(0,0,0), innov = inn*0.0123)
	
	tst = cbind(head(as.data.frame(sim1)), head(as.data.frame(sim2)), head(as.data.frame(sim3)))
	colnames(tst) = c("ARFIMA(d = 0)", "ARMA", "arima.sim")
	
	zz <- file("test13-7-2.txt", open="wt")
	sink(zz)
	cat("\nARFIMA, ARMA arima.sim simulation tests:\n")
	print(tst, digits = 6)
	sink(type="message")
	sink()
	close(zz)
}