#################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################

#################################################################################
# Fitting tests
#################################################################################

rgarch.test21 = function()
{
	cat("\nrgarch-->test2-1: Fit Test (fGARCH)\n")
	
	data(dji30ret)
	# 1. GARCH
	spec = ugarchspec(variance.model = list(model = "fGARCH", garchOrder = c(1,1), submodel = "GARCH"), mean.model = list(armaOrder = c(1,1), include.mean = TRUE), distribution.model = "std")
	
	fgarch.s = ugarchfit(data = dji30ret[,"AA",drop=FALSE], spec = spec, solver = "solnp")
	
	# 2. TGARCH
	spec = ugarchspec(variance.model = list(model = "fGARCH", garchOrder = c(1,1), submodel = "TGARCH"), mean.model = list(armaOrder = c(1,1), include.mean = TRUE), distribution.model = "std")
	
	fgarch.t = ugarchfit(data = dji30ret[,"AA",drop=FALSE], spec = spec, solver = "solnp")
	
	# 3. NGARCH
	spec = ugarchspec(variance.model = list(model = "fGARCH", garchOrder = c(1,1),  submodel = "NGARCH"), mean.model = list(armaOrder = c(1,1), include.mean = TRUE), distribution.model = "std")
	
	fgarch.n = ugarchfit(data = dji30ret[,"AA",drop=FALSE], spec = spec, solver = "solnp")
	
	
	# 4. NAGARCH
	spec = ugarchspec(variance.model = list(model = "fGARCH", garchOrder = c(1,1),  submodel = "NAGARCH"), mean.model = list(armaOrder = c(1,1), include.mean = TRUE), distribution.model = "std")
	
	fgarch.na = ugarchfit(data = dji30ret[,"AA",drop=FALSE], spec = spec, solver = "solnp")
	
	# 5. AVGARCH
	spec = ugarchspec(variance.model = list(model = "fGARCH", garchOrder = c(1,1), submodel = "AVGARCH"), mean.model = list(armaOrder = c(1,1), include.mean = TRUE), distribution.model = "std")
	
	fgarch.av = ugarchfit(data = dji30ret[,"AA",drop=FALSE], spec = spec, solver = "solnp")
	
	# 6. APARCH
	spec = ugarchspec(variance.model = list(model = "fGARCH", garchOrder = c(1,1),  submodel = "APARCH"), mean.model = list(armaOrder = c(1,1), include.mean = TRUE), distribution.model = "std")
	
	fgarch.ap = ugarchfit(data = dji30ret[,"AA",drop=FALSE], spec = spec, solver = "solnp")
	
	# 7. ALLGARCH (needs some starting values, we use aparch fit)
	spec = ugarchspec(variance.model = list(model = "fGARCH", garchOrder = c(1,1), submodel = "ALLGARCH"), mean.model = list(armaOrder = c(1,1), include.mean = TRUE), distribution.model = "std", start.pars = as.list(coef(fgarch.ap)))
	
	fgarch.al = ugarchfit(data = dji30ret[,"AA",drop=FALSE], spec = spec, solver = "solnp")
	
	# compare coefficients and likelihoods
	fgarch.lik = c(likelihood(fgarch.s), likelihood(fgarch.t) ,likelihood(fgarch.n), likelihood(fgarch.na), likelihood(fgarch.av), likelihood(fgarch.ap), likelihood(fgarch.al))
	
	fgarch.cnames = c("mu", "ar1", "ma1", "omega", "alpha1", "gamma11", "gamma21", "beta1", "lambda", "shape")

	fgarch.models = matrix(NA, ncol = 7, nrow = 10)
	fgarch.models[,1] = coef(fgarch.s)[fgarch.cnames]
	fgarch.models[,2] = coef(fgarch.t)[fgarch.cnames]
	fgarch.models[,3] = coef(fgarch.n)[fgarch.cnames]
	fgarch.models[,4] = coef(fgarch.na)[fgarch.cnames]
	fgarch.models[,5] = coef(fgarch.av)[fgarch.cnames]
	fgarch.models[,6] = coef(fgarch.ap)[fgarch.cnames]
	fgarch.models[,7] = coef(fgarch.al)[fgarch.cnames]
	fgarch.models = rbind(fgarch.models, fgarch.lik)

	colnames(fgarch.models) = c("GARCH", "TGARCH", "NGARCH", "NAGARCH", "AVGARCH", "APARCH", "ALLGARCH")

	rownames(fgarch.models) = c(fgarch.cnames, "LLH")
	fgarch.models = fgarch.models[,order(fgarch.models[11,])]
	options(width=150)
	zz <- file("test2-1.txt", open="wt")
	sink(zz)
	print(round(fgarch.models,5), digits = 6)
	sink(type="message")
	sink()
	close(zz)
}

#test2-1.txt


# ---------------------------------------------------------------------------------
# Models with variation in mean model choices and external regressors
# in mean and variance equation (fully parametrized model - we follow an
# incremental building strategy using the coefficients from previous fits
# as starting parameters into more complicated models)

rgarch.test22 = function()
{
	cat("\nrgarch-->test2-2: Fit Test (sGARCH)\n")
	data(dji30ret)
	dates = rownames(dji30ret[,"AA", drop = FALSE])
	monday = WeekDayDummy(dates, date.format = "%Y-%m-%d", weekday = "Monday")
	# convert to matrix which is what the specification expects
	monday = matrix(monday, ncol = 1)
	# create a dummy day-of-week variable for the variance regression (Friday)
	friday = WeekDayDummy(dates, date.format = "%Y-%m-%d", weekday = "Friday")
	# convert to matrix which is what the specification expects
	friday = matrix(friday, ncol = 1)
	# sGARCH(1,1)
	spec = ugarchspec(variance.model = list(model = "sGARCH", garchOrder = c(1,1)), mean.model = list(armaOrder = c(0,0), include.mean = FALSE), distribution.model = "std")
	
	sgarch.fit1 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# sGARCH(1,1) + MU
	spec = ugarchspec(variance.model = list(model = "sGARCH", garchOrder = c(1,1)), mean.model = list(armaOrder = c(0,0), include.mean = TRUE), distribution.model = "std")
	
	sgarch.fit2 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# sGARCH(1,1) + MU + ARMA(1,1)
	spec = ugarchspec(variance.model = list(model = "sGARCH", garchOrder = c(1,1)), mean.model = list(armaOrder = c(1,1), include.mean = TRUE), distribution.model = "std")
	
	sgarch.fit3 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# sGARCH(1,1) + XREG(V) + ARMA(1,1) + XREG(M)
	spec = ugarchspec(variance.model = list(model = "sGARCH", garchOrder = c(1,1), external.regressors = friday), mean.model = list(armaOrder = c(1,1), include.mean = FALSE, external.regressors = monday), distribution.model = "std", start.pars = as.list(coef(sgarch.fit3)))
	
	sgarch.fit4 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# sGARCH(1,1) + XREG(V) + MU + ARMA(1,1) + XREG(M)
	spec = ugarchspec(variance.model = list(model = "sGARCH", garchOrder = c(1,1), external.regressors = friday), mean.model = list(armaOrder = c(1,1), include.mean = TRUE, external.regressors = monday), distribution.model = "std", start.pars = as.list(coef(sgarch.fit4 )))
	
	sgarch.fit5 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# sGARCH(1,1) + XREG(V) + ARMA(1,1) + XREG(M) + INMEAN(1)
	spec = ugarchspec(variance.model = list(model = "sGARCH", garchOrder = c(1,1), external.regressors = friday), mean.model = list(armaOrder = c(1,1), include.mean = TRUE, garchInMean = TRUE, inMeanType = 1, external.regressors = monday), distribution.model = "std", start.pars = as.list(coef(sgarch.fit5)))
	
	sgarch.fit6 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# compare coefficients and likelihoods
	sgarch.lik = c(likelihood(sgarch.fit1), likelihood(sgarch.fit2) ,likelihood(sgarch.fit3), likelihood(sgarch.fit4), likelihood(sgarch.fit5), likelihood(sgarch.fit6))
	
	sgarch.cnames = c("mu", "ar1", "ma1", "inmean", "mxreg1", "omega", "alpha1", "beta1", "vxreg1", "shape")
	
	sgarch.models = matrix(NA, ncol = 6, nrow = 10)
	sgarch.models[,1] = coef(sgarch.fit1)[sgarch.cnames]
	sgarch.models[,2] = coef(sgarch.fit2)[sgarch.cnames]
	sgarch.models[,3] = coef(sgarch.fit3)[sgarch.cnames]
	sgarch.models[,4] = coef(sgarch.fit4)[sgarch.cnames]
	sgarch.models[,5] = coef(sgarch.fit5)[sgarch.cnames]
	sgarch.models[,6] = coef(sgarch.fit6)[sgarch.cnames]
	sgarch.models = rbind(sgarch.models, sgarch.lik)
	
	colnames(sgarch.models) = c("model1", "model2", "model3", "model4", "model5", "model6")
	rownames(sgarch.models) = c(sgarch.cnames, "LLH")
	options(width=150)
	zz <- file("test2-2.txt", open="wt")
	sink(zz)
	print(round(sgarch.models[,order(sgarch.models[11,])],5), digits = 6)
	sink(type="message")
	sink()
	close(zz)
}


#test2-2.txt


# ---------------------------------------------------------------------------------
# apARCH
# ---------------------------------------------------------------------------------
rgarch.test23 = function()
{
	cat("\nrgarch-->test2-3: Fit Test (apARCH)\n")
	
	data(dji30ret)
	dates = rownames(dji30ret[,"AA", drop = FALSE])
	monday = WeekDayDummy(dates, date.format = "%Y-%m-%d", weekday = "Monday")
	# convert to matrix which is what the specification expects
	monday = matrix(monday, ncol = 1)
	# create a dummy day-of-week variable for the variance regression (Friday)
	friday = WeekDayDummy(dates, date.format = "%Y-%m-%d", weekday = "Friday")
	# convert to matrix which is what the specification expects
	friday = matrix(friday, ncol = 1)
	# apARCH(1,1)
	spec = ugarchspec(variance.model = list(model = "apARCH", garchOrder = c(1,1)), mean.model = list(armaOrder = c(0,0), include.mean = FALSE), distribution.model = "std")
	
	aparch.fit1 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# apARCH(1,1) + MU
	spec = ugarchspec(variance.model = list(model = "apARCH", garchOrder = c(1,1)), mean.model = list(armaOrder = c(0,0), include.mean = TRUE), distribution.model = "std")
	
	aparch.fit2 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# apARCH(1,1) + MU + ARMA(1,1)
	spec = ugarchspec(variance.model = list(model = "apARCH", garchOrder = c(1,1)), mean.model = list(armaOrder = c(1,1), include.mean = TRUE), distribution.model = "std")
	
	aparch.fit3 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# apARCH(1,1) + XREG(V) + ARMA(1,1) + XREG(M)
	spec = ugarchspec(variance.model = list(model = "apARCH", garchOrder = c(1,1), external.regressors = friday), mean.model = list(armaOrder = c(1,1), include.mean = FALSE, external.regressors = monday), distribution.model = "std", start.pars = as.list(coef(aparch.fit3 )))
	
	aparch.fit4 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# apARCH(1,1) + XREG(V) + MU + ARMA(1,1) + XREG(M)
	spec = ugarchspec(variance.model = list(model = "apARCH", garchOrder = c(1,1), external.regressors = friday), mean.model = list(armaOrder = c(1,1), include.mean = TRUE, external.regressors = monday), distribution.model = "std", start.pars = as.list(coef(aparch.fit4 )))
	
	aparch.fit5 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# apARCH(1,1) + XREG(V) + ARMA(1,1) + XREG(M) + INMEAN(1)
	spec = ugarchspec(variance.model = list(model = "apARCH", garchOrder = c(1,1), external.regressors = friday), mean.model = list(armaOrder = c(1,1), include.mean = TRUE, garchInMean = TRUE, inMeanType = 1, external.regressors = monday), distribution.model = "std", start.pars = as.list(coef(aparch.fit5)))
	
	aparch.fit6 = ugarchfit(data=dji30ret[,"AA", drop = FALSE], spec = spec, solver = "solnp")
	
	# compare coefficients and likelihoods
	aparch.lik = c(likelihood(aparch.fit1), likelihood(aparch.fit2) ,likelihood(aparch.fit3), likelihood(aparch.fit4), likelihood(aparch.fit5), likelihood(aparch.fit6))
	
	aparch.cnames = c("mu", "ar1", "ma1", "inmean", "mxreg1", "omega", "alpha1", "beta1", "gamma1", "delta", "vxreg1", "shape")
	
	aparch.models = matrix(NA, ncol = 6, nrow = 12)
	aparch.models[,1] = coef(aparch.fit1)[aparch.cnames]
	aparch.models[,2] = coef(aparch.fit2)[aparch.cnames]
	aparch.models[,3] = coef(aparch.fit3)[aparch.cnames]
	aparch.models[,4] = coef(aparch.fit4)[aparch.cnames]
	aparch.models[,5] = coef(aparch.fit5)[aparch.cnames]
	aparch.models[,6] = coef(aparch.fit6)[aparch.cnames]
	aparch.models = rbind(aparch.models, aparch.lik)
	
	colnames(aparch.models) = c("model1", "model2", "model3", "model4", "model5", "model6")
	rownames(aparch.models) = c(aparch.cnames, "LLH")
	
	options(width=150)
	zz <- file("test2-3.txt", open="wt")
	sink(zz)
	print(round(aparch.models[,order(aparch.models[13,])],5), digits = 6)
	sink(type="message")
	sink()
	close(zz)
}

#test2-3.txt
