#################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################

#################################################################################
# GARCH rolling test
#################################################################################


rgarch.test71 = function()
{
	cat("\nrgarch-->test7-1: Roll Test (apARCH)\n")	
	data(sp500ret)
	spec = ugarchspec(variance.model = list(model = "apARCH"), distribution = "nig")
	roll = ugarchroll(spec,  data = sp500ret, n.ahead = 1, forecast.length = 200, 
			refit.every = 25, refit.window = "recursive", use.multicore = FALSE, 
			solver = "solnp", fit.control = list(scale = 0), solver.control = list() ,
			calculate.VaR = TRUE, VaR.alpha = c(0.01, 0.05))
	postscript("test7-1-1.eps", width = 12, height = 8)
	plot(roll, which = "all")
	dev.off()
	
	postscript("test7-1-2.eps", width = 12, height = 8)
	plot(roll, which = 5)
	dev.off()
	
	zz <- file("test7-1-1.txt", open="wt")
	sink(zz)
	report(roll, type = "VaR", n.ahead = 1, VaR.alpha = 0.01, conf.level = 0.95)
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test7-1-2.txt", open="wt")
	sink(zz)
	report(roll, type = "fpm")
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test7-1-3.txt", open="wt")
	sink(zz)
	as.data.frame(roll, which = "coefs")
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test7-1-4.txt", open="wt")
	sink(zz)
	as.data.frame(roll, which = "density")
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test7-1-5.txt", open="wt")
	sink(zz)
	as.data.frame(roll, which = "coefmat", refit = 1)
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test7-1-6.txt", open="wt")
	sink(zz)
	as.data.frame(roll, which = "VaR")
	sink(type="message")
	sink()
	close(zz)
}

rgarch.test72 = function()
{
	cat("\nrgarch-->test7-2: Roll Test (apARCH)\n")	
	data(sp500ret)
	spec = ugarchspec(variance.model = list(model = "apARCH"), distribution = "sstd")
	roll = ugarchroll(spec,  data = sp500ret, n.ahead = 5, forecast.length = 200, 
			refit.every = 25, refit.window = "recursive", use.multicore = FALSE, 
			solver = "solnp", fit.control = list(scale = 1), solver.control = list(scale = 0) ,
			calculate.VaR = TRUE, VaR.alpha = c(0.01, 0.05))
	
	postscript("test7-2-1.eps", width = 12, height = 8)
	plot(roll, n.ahead = 5, which = "all")
	dev.off()
	
	postscript("test7-2-2.eps", width = 12, height = 8)
	plot(roll, which = 5)
	dev.off()
	
	zz <- file("test7-2-1.txt", open="wt")
	sink(zz)
	report(roll, type = "VaR", n.ahead = 1, VaR.alpha = 0.01, conf.level = 0.95)
	report(roll, type = "VaR", n.ahead = 5, VaR.alpha = 0.01, conf.level = 0.95)
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test7-2-2.txt", open="wt")
	sink(zz)
	report(roll, type = "fpm", n.ahead = 1)
	report(roll, type = "fpm", n.ahead = 5)
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test7-2-3.txt", open="wt")
	sink(zz)
	as.data.frame(roll, which = "coefs")
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test7-2-4.txt", open="wt")
	sink(zz)
	as.data.frame(roll, which = "density", n.ahead = 1)
	as.data.frame(roll, which = "density", n.ahead = 5)
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test7-2-5.txt", open="wt")
	sink(zz)
	as.data.frame(roll, which = "coefmat", refit = 1)
	as.data.frame(roll, which = "coefmat", refit = 8)
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test7-2-6.txt", open="wt")
	sink(zz)
	as.data.frame(roll, which = "VaR", n.ahead = 1)
	as.data.frame(roll, which = "VaR", n.ahead = 5)
	sink(type="message")
	sink()
	close(zz)
}
