#################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################

#################################################################################
# GARCH distribution tests
#################################################################################
rgarch.seeds = function(n, test)
{
	rseed = round(runif(n,100,10000),0)
	options(width=120)
	zz <- file(paste(test,"-seeds.txt", sep=""), open="wt")
	sink(zz)
	print(rseed)
	sink(type="message")
	sink()
	close(zz)
	return(rseed)
}
# non recursive, simple test

rgarch.test81 = function()
{
	cat("\nrgarch-->test8-1: Parameter Distribution Test - Non recursive (fGARCH/NAGARCH)\n")	
	data(dji30ret)
	rseed = rgarch.seeds(100, "test8-1")
	spec = ugarchspec(variance.model = list(model = "fGARCH", submodel = "NAGARCH"), distribution.model = "std")
	fit = ugarchfit(spec, data = dji30ret[,"XOM"], fit.control = list(scale = 1))
	dist = ugarchdistribution(fit, n.sim = 2000, n.start = 50, m.sim = 4, solver = "solnp", 
			fit.control = list(scale = 1), rseed  = rseed)
	postscript("test8-1-1.eps", width = 12, height = 8)
	plot(dist, which = 1)
	dev.off()
	
	postscript("test8-1-2.eps", width = 12, height = 8)
	plot(dist, which = 2)
	dev.off()
	
	postscript("test8-1-3.eps", width = 12, height = 8)
	plot(dist, which = 3)
	dev.off()
	
	zz <- file("test8-1-1.txt", open="wt")
	sink(zz)
	as.data.frame(dist, which = "coef")
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test8-1-2.txt", open="wt")
	sink(zz)
	as.data.frame(dist, which = "rmse")
	sink(type="message")
	sink()
	close(zz)
	
	
	zz <- file("test8-1-3.txt", open="wt")
	sink(zz)
	as.data.frame(dist, which = "stats")
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test8-1-4.txt", open="wt")
	sink(zz)
	as.data.frame(dist, which = "coefse")
	sink(type="message")
	sink()
	close(zz)
}

# recursive test and asymptotic consistency
rgarch.test82 = function()
{
	cat("\nrgarch-->test8-2: Parameter Distribution Test - recursive (fGARCH/NAGARCH)\n")	
	data(dji30ret)
	spec = ugarchspec(variance.model = list(model = "fGARCH", submodel = "NAGARCH"), distribution.model = "std")
	fit = ugarchfit(spec, data = dji30ret[,"XOM"], solver.control = list(scale = 1))
	dist = ugarchdistribution(fit, n.sim = 1000, n.start = 50, m.sim = 100, 
			recursive = TRUE, recursive.length = 6000, recursive.window = 1000)
	save(dist, file = "dist.rda")
	
	jpeg("test8-2-1.jpeg", width = 12, height = 8, units = "in", res = 100)
	plot(dist, which = 1)
	dev.off()
	
	jpeg("test8-2-2.jpeg", width = 12, height = 8, units = "in", res = 100)
	plot(dist, which = 2, window = 6)
	dev.off()
	
	jpeg("test8-2-3.jpeg", width = 12, height = 8, units = "in", res = 100)
	plot(dist, which = 3)
	dev.off()
	
	jpeg("test8-2-4.jpeg", width = 12, height = 8, units = "in", res = 100)
	plot(dist, which = 4)
	dev.off()
	
	zz <- file("test8-2-1.txt", open="wt")
	sink(zz)
	as.data.frame(dist, which = "coef", window = 6)
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test8-2-2.txt", open="wt")
	sink(zz)
	rbind(as.data.frame(dist, which = "rmse", window = 1), 
			as.data.frame(dist, which = "rmse", window = 2),
			as.data.frame(dist, which = "rmse", window = 3),
			as.data.frame(dist, which = "rmse", window = 4),
			as.data.frame(dist, which = "rmse", window = 5),
	as.data.frame(dist, which = "rmse", window = 6))
	sink(type="message")
	sink()
	close(zz)
	
	
	zz <- file("test8-2-3.txt", open="wt")
	sink(zz)
	as.data.frame(dist, which = "stats", window = 6)
	sink(type="message")
	sink()
	close(zz)
	
	zz <- file("test8-2-4.txt", open="wt")
	sink(zz)
	as.data.frame(dist, which = "coefse", window = 6)
	sink(type="message")
	sink()
	close(zz)
}
