#################################################################################
##
##   R package rgarch by Alexios Ghalanos Copyright (C) 2009
##   This file is part of the R package rgarch.
##
##   The R package rgarch is free software: you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   The R package rgarch is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
#################################################################################

#################################################################################
# GARCH bootstrap prediction tests
#################################################################################

# partial bootstrap
rgarch.test91 = function()
{
	cat("\nrgarch-->test9-1: Bootstrap Prediction Test - partial method (apARCH)\n")	
	data(dji30ret)
	spec1 = ugarchspec(variance.model = list(model = "apARCH"), distribution = "std")
	fit = ugarchfit(spec1, data = dji30ret[,"AA", drop = FALSE], fit.control = list(scale = 1),
			out.sample = 150)
	spec2 = spec1
	spec2@optimization.model$fixed.pars = as.list(coef(fit))
	bt1 = ugarchboot(fit, method = "Partial", n.ahead = 125, n.bootpred = 4000, rseed = 1:4000)
	bt2 = ugarchboot(spec2, data = dji30ret[,"AA", drop = FALSE], 
			out.sample = 150, method = "Partial", n.ahead = 125, n.bootpred = 4000, rseed = 1:4000)
	
	# check that the 2 methods coincide
	zz <- file("test9-1.txt", open="wt")
	sink(zz)
	cat("\nPartial Bootstrap (fit method)\n")
	(bt1)
	cat("\nPartial Bootstrap (spec method)\n")
	(bt2)
	sink(type="message")
	sink()
	close(zz)
	
	postscript("test9-1.eps", width = 12, height = 8)
	plot(bt1, which="all")
	dev.off()
}



# full bootstrap
rgarch.test92 = function()
{
	cat("\nrgarch-->test9-2: Bootstrap Prediction Test - full method (apARCH)\n")	
	data(dji30ret)
	spec1 = ugarchspec(variance.model = list(model = "apARCH"), distribution = "std")
	fit = ugarchfit(spec1, data = dji30ret[,"AA", drop = FALSE], fit.control = list(scale = 1),
			out.sample = 150)
	bt = ugarchboot(fit, method = "full", n.ahead = 25, n.bootfit = 100, n.bootpred = 4000,
			rseed = 1:4100)
	
	
	spec2 = spec1
	spec2@optimization.model$fixed.pars = as.list(coef(fit))
	bt2 = ugarchboot(spec2, data = dji30ret[,"AA", drop = FALSE], out.sample = 150, method = "full", 
			n.ahead = 25, n.bootfit = 100, n.bootpred = 4000, rseed = 1:4100)
	
	zz <- file("test9-2.txt", open="wt")
	sink(zz)
	cat("\nFull Bootstrap (fit method)\n")
	bt
	cat("\nFull Bootstrap (spec method)\n")
	bt2
	sink(type="message")
	sink()
	close(zz)
	
	postscript("test9-2-1.eps", width = 12, height = 8)
	plot(bt2, which=1)
	dev.off()

	postscript("test9-2-2.eps", width = 12, height = 8)
	plot(bt2, which="all")
	dev.off()
}
